@@ -1,20 +1,20 @@
 GWAS <- function(fixed, random, rcov, data, weights, W,
-                 nIters=20, tolParConvLL = 1e-03, tolParInv = 1e-06, 
-                 init=NULL, constraints=NULL, method="NR", 
+                 nIters=20, tolParConvLL = 1e-03, tolParInv = 1e-06,
+                 init=NULL, constraints=NULL, method="NR",
                  getPEV=TRUE,
                  naMethodX="exclude",
                  naMethodY="exclude",
-                 returnParam=FALSE, 
+                 returnParam=FALSE,
                  dateWarning=TRUE,date.warning=TRUE,
-                 verbose=FALSE, 
+                 verbose=FALSE,
                  stepWeight=NULL, emWeight=NULL,
-                 M=NULL, gTerm=NULL, n.PC = 0, min.MAF = 0.05, 
+                 M=NULL, gTerm=NULL, n.PC = 0, min.MAF = 0.05,
                  P3D = TRUE){
-  
+
   if(is.null(gTerm)){
     stop("Please provide the name of the genetic term in the model in the gTerm argument", call. = FALSE)
   }
-  
+
   if(length(which(is.na(M))) > 0){
     stop("Please provide an imputed marker matrix (M).", call. = FALSE)
   }
@@ -23,24 +23,26 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
     r <- max(which(svd.X$d > 1e-08))
     return(as.matrix(svd.X$u[, 1:r]))
   }
-  
+
   dwToUse <- c(dateWarning,date.warning)
   v<-which(!dwToUse)
   if(length(v) == 0){v<-1}
+  # print("hey")
   ## return all parameters for a mixed model (NOT ACTUAL FITTING)
-  res <- mmer(fixed=fixed, random=random, rcov=rcov, data=data, weights=weights, W=W,  # silence weights and W when testing
-              nIters=nIters, tolParConvLL=tolParConvLL, tolParInv=tolParInv, 
-              init=init, constraints=constraints, method=method, 
+  res <- mmer(fixed=fixed, random=random, rcov=rcov, data=data, W=W,  # silence weights and W when testing
+              nIters=nIters, tolParConvLL=tolParConvLL, tolParInv=tolParInv,
+              init=init, constraints=constraints, method=method,
               getPEV=getPEV,
               naMethodX=naMethodX,
               naMethodY=naMethodY,
-              returnParam=TRUE, 
+              returnParam=TRUE,
               dateWarning=dwToUse[v],
               verbose=verbose,
               stepWeight=stepWeight, emWeight=emWeight
               )
+  # print("hey")
   # print(str(res))
-  
+
   if(returnParam){ # if user only wants the initial parameters and objects
     lastmodel <- res
   }else{  # if actual fitting is required
@@ -59,9 +61,9 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
         stop("No match between the Gterm and the random effects present.")
       }
       ## get input matrices
-      Y <- scale(res$yvar) 
+      Y <- scale(res$yvar)
       Z <- do.call(cbind,res$Z[gTermi])
-      
+
       if (n.PC > 0) {
         Kb <- do.call(adiag1,res$K[gTermi])
         eig.vec <- eigen(Kb)$vectors
@@ -72,11 +74,11 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
         X <- do.call(cbind,res$X)
         X <- make.full(X)
       }
-      
+
       if(nrow(M) != ncol(Z)){
         stop(paste("Marker matrix M needs to have same numbers of rows(",nrow(M),") than columns of the gTerm incidence matrix(",ncol(Z),")."),call. = FALSE)
       }
-      
+
       if(length(which(rownames(M) != colnames(Z)))){
         M <- M[colnames(Z),]
       }
@@ -87,19 +89,23 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
       cat(red("Performing GWAS evaluation\n"))
       preScores <- .Call("_sommer_gwasForLoop",PACKAGE = "sommer",
                          M,Y,as.matrix(Z),X,Vinv,min.MAF,TRUE
-      ) 
+      )
       v2 <- length(Y) - ((ncol(X)+1)*ncol(Y)) # ncol(XZMi)
-      scores <- -log10(pbeta(preScores, v2/2, 1/2))
+      pvals <- pbeta(preScores, v2/2, 1/2)
+      scores <- -log10(pvals)
       ########################
       rownames(scores) <- colnames(M)
       colnames(scores) <- colnames(Y)
       
+      lastmodel$pvals <- pvals
       lastmodel$scores <- scores
+      lastmodel$shape1 <- v2/2
+      lastmodel$shape2 <- 1/2
       lastmodel$method <- method
       lastmodel$constraints <- res[[8]]
       class(lastmodel)<-c("mmergwas")
     }else{ # if different variance for each marker
-      
+
       ## get names of random effects
       re_names <- unlist(res$re_names)
       re_names <- gsub("\\(Intercept):","",re_names)
@@ -107,12 +113,13 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
       if(length(gTermi) < 1){
         stop("No match between the Gterm and the random effects present.")
       }
-      Y <- scale(res$yvar) 
+      Y <- scale(res$yvar)
+      # print(head(Y))
       Z <- do.call(cbind,res$Z[gTermi])
       if(nrow(M) != ncol(Z)){
         stop(paste("Marker matrix M needs to have same numbers of rows(",nrow(M),") than columns of the gTerm incidence matrix(",ncol(Z),")."),call. = FALSE)
       }
-      
+
       if (n.PC > 0) {
         Kb <- do.call(adiag1,res$K[gTermi])
         eig.vec <- eigen(Kb)$vectors
@@ -124,9 +131,9 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
         X <- make.full(X)
       }
       #
-      preScores <- bs <- numeric()
+      preScores <- bs <- list()
       for(iMarker in 1:ncol(M)){
-        print(iMarker)
+        # print(iMarker)
         mi <- M[,iMarker]
         if(var(mi) > 0){ # if var > 0
           Xmi <- cbind(X,mi)
@@ -138,24 +145,35 @@ GWAS <- function(fixed, random, rcov, data, weights, W,
           var.b = diag(as.matrix(lastmodel$VarBeta))
           se.b = sqrt(abs(var.b))
           t.val  = b/se.b
-          preScores[iMarker] = 1 - pnorm(t.val[nrow(t.val),])
-          bs[iMarker] = b[nrow(b),] # last fixed effect where we put the marker
+          # print(t.val)
+          preScores[[iMarker]] = 1 - pnorm(t.val[(nrow(t.val)-ncol(Y)+1):nrow(t.val),])
+          # print(b)
+          bs[[iMarker]] = b[(nrow(b)-ncol(Y)+1):nrow(b),] # last fixed effect where we put the marker
         }else{ # if var == 0
-          preScores[iMarker] <- 1
-          bs[iMarker] = 0 # effect
+          preScores[[iMarker]] <- rep(1,ncol(Y))
+          bs[[iMarker]] = rep(0,ncol(Y)) # effect
         }
       } # for loop for each marker
-      scores <- as.matrix(-log10(preScores))
+      preScores <- do.call(rbind,preScores)
+      bs <- do.call(rbind, bs)
+      scores <- apply(preScores, 2, function(x){-log10(x)})
+      
+      # scores <- as.matrix(-log10(preScores))
+      
       rownames(scores) <- colnames(M) # marker names
+      # print(head(scores));print(colnames(Y))
       colnames(scores) <- colnames(Y) # trait names
+     
+      lastmodel$effects <- bs
       lastmodel$scores <- scores
+      lastmodel$pvals <- preScores
       lastmodel$method <- method
       lastmodel$constraints <- res[[8]]
       class(lastmodel)<-c("mmergwas")
-      
+
     }
-    
+
   } # if actual fitting is required
-  
+
   return(lastmodel)
 }
